//----------------------------------------------------------------------------------------
// UI Events protocol
//----------------------------------------------------------------------------------------

#ifndef _VIDEOPART_
#include "CPVideoPart.h"
#endif
#ifndef SOM_ODDispatcher_xh
#include <Disptch.xh>
#endif

static pascal Boolean MySettingsDialogFilter (DialogPtr dialog, EventRecord *event,
											  short *itemHit);

//----------------------------------------------------------------------------------------

ODBoolean CPVideoPart::HandleEvent(Environment* ev, ODEventData* event,
								   ODFrame* frame, ODFacet* facet,
								   ODEventInfo* eventInfo)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_HandleEvent
	EnteringMethod("\pCPVideoPart::HandleEvent");
	ODUnused(eventInfo);

	ODBoolean tWasHandled = kODFalse;

	switch (event->what) {
		case nullEvent:
			if (::IsDialogEvent((EventRecord *)event))
			{
				short itemHit;
				DialogPtr dialog;
				tWasHandled = ::DialogSelect((EventRecord *)event, &dialog, &itemHit);
				return tWasHandled;
			}
			else if (frame) {

				//	We dont need to redraw the image if the frame is a large icon or
				//	small icon view.
				
				ODTypeToken view;
				
				view = frame->GetViewType (ev);
				if (view == fSession->Tokenize (ev, kODViewAsThumbnail) ||
					view == fSession->Tokenize (ev, kODViewAsFrame)) {
					ODFacet *facet;
					ODFrameFacetIterator *facetIterator;
	
					//	Update the offscreen image (i.e. grab the latest frame in our GWorld).
	
					SGIdle (fSequenceGrabberComponent);
	
					//	Draw the image into each of the frames facets.
					
					facetIterator = frame->CreateFacetIterator (ev);
					facet = facetIterator->First (ev);
					while (facetIterator->IsNotComplete (ev)) {
						RgnHandle newClip, oldClip;
					
						CFocus foc (ev, facet);
						
						//	If were the root, clip the drawing area so we dont clobber the
						//	windows grow box. Otherwise CFocus already set everything up so we
						//	dont need to do anything.
						
						oldClip = nil;
						if (frame->IsRoot (ev)) {
							GrafPtr	currentGrafPort;
							Rect growIconRect;
					
							::GetPort (&currentGrafPort);
							growIconRect = currentGrafPort->portRect;
							growIconRect.left = growIconRect.right  - 15;
							growIconRect.top  = growIconRect.bottom - 15;
							::GetClip (oldClip = NewRgn ());
							::RectRgn (newClip = ::NewRgn (), &growIconRect);
							::DiffRgn (oldClip, newClip, newClip);
							::SetClip (newClip);
							::DisposeRgn (newClip);
						}
						
						//	Now actually draw the content.
						
						if (view == fSession->Tokenize (ev, kODViewAsThumbnail))
							PrivDrawAsThumbnail (ev, facet);
						else
							PrivDrawContent (ev, facet);
						
						//	Restore the old clip region.
						
						if (oldClip) {
							::SetClip (oldClip);
							::DisposeRgn (oldClip);
						}
						
						//	Do the next facet.
						
						facet = facetIterator->Next (ev);
					}
				}
			}
			tWasHandled = kODTrue;
			break;
		case mouseDown:
			tWasHandled = this->PrivHandleMouseDown (ev, event, frame, facet);
			break;
		case kODEvtMouseDownEmbedded:
		case mouseUp:
		case keyDown:
		case autoKey:
		case activateEvt:
			tWasHandled = kODTrue;
			break;
		case kODEvtMenu:
			tWasHandled = this->PrivHandleMenuEvent (ev, frame, event);
			break;
		default:
			break;
	}
	return tWasHandled;
}

//----------------------------------------------------------------------------------------

void CPVideoPart::AdjustMenus(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_AdjustMenus
	EnteringMethod("\pCPVideoPart::AdjustMenus");
	ODUnused(frame);

	Str255		aboutText;
	ODIText*	odiText;

	fMenuBar->EnableCommand(ev, kODCommandViewAsWin, !frame->IsRoot(ev) );
	fMenuBar->EnableCommand(ev, kODCommandGetPartInfo, kODTrue);

	if (fSequenceGrabberComponent)
		fMenuBar->EnableCommand (ev, kSettingsCommand, kODTrue);
	else
		fMenuBar->EnableCommand (ev, kSettingsCommand, kODFalse);

	{
		CUsingLibraryResources fil;
		::GetIndString (aboutText, kSTRx_MenuItemStrings, kAboutMenuItemString);
	}

	ODScriptCode script = ::GetScriptManagerVariable(smSysScript);
	ODLangCode lang = ::GetScriptVariable(script, smScriptLang);
	odiText = CreateIText(script, lang, aboutText);
	fMenuBar->SetItemString(ev, kODCommandAbout, odiText);
	DisposeIText(odiText);
}

//----------------------------------------------------------------------------------------

ODSession* gSession;

ODBoolean CPVideoPart::PrivHandleMenuEvent(Environment* ev, ODFrame* frame, ODEventData* event)
{
	EnteringMethod("\pCPVideoPart::PrivHandleMenuEvent");
	ODUnused(frame);

	long menuResult = event->message;
	short menu = HiWord (menuResult);
	short item = LoWord (menuResult);

	if (menu) {
		switch (fMenuBar->GetCommand (ev, menu, item)) {
			case kODCommandAbout:
				{
					CUsingLibraryResources fil;
		
					if (fSession->GetArbitrator(ev)->RequestFocus (ev, fModalFocus, frame)) {
						::SetCursor (&ODQDGlobals.arrow);
						gSession = fSession;				// DialogFilter uses this!
						ModalFilterUPP modalFilter = NewModalFilterProc (MyDialogFilter);
						::Alert (kVideoPartAboutBoxID, modalFilter);
						DisposeRoutineDescriptor (modalFilter);
						fSession->GetArbitrator (ev)->RelinquishFocus (ev, fModalFocus, frame);
					}
					else
						::SysBeep(0);
				}
				break;
			case kODCommandViewAsWin:
				this->Open(ev, frame);
				break;
			case kODCommandGetPartInfo:
				ODFrameFacetIterator* facets = frame->CreateFacetIterator(ev);
				fSession->GetInfo(ev)->ShowPartFrameInfo(ev, facets->First(ev), kODFalse);
				break;
			case kSettingsCommand:
				{
					CUsingLibraryResources fil;
	
					if (fSession->GetArbitrator (ev)->RequestFocus (ev, fModalFocus, frame)) {
						ModalFilterUPP modalFilter;
						
						::SetCursor (&ODQDGlobals.arrow);
						gSession = fSession;
						modalFilter = NewModalFilterProc (MySettingsDialogFilter);
						::SGSettingsDialog (fSequenceGrabberComponent, fVideoChannel, 0, nil,
											0, modalFilter, 0);
						DisposeRoutineDescriptor (modalFilter);
						fSession->GetArbitrator (ev)->RelinquishFocus (ev, fModalFocus,frame);
					}
					else
						::SysBeep (0);
				}
				break;
			default:
				return kODFalse;
		}
	}
	return kODTrue;
}

//------------------------------------------------------------------------------

static pascal Boolean MySettingsDialogFilter (DialogPtr dialog, EventRecord *event,
											  short *itemHit)
{
	Boolean handled = false;
	
	if (event->what == nullEvent ||
		event->what == updateEvt && (WindowPtr) event->message != dialog ||
		event->what == activateEvt && (WindowPtr) event->message != dialog) {
		Environment *ev = somGetGlobalEnvironment ();
		gSession->GetDispatcher (ev)->Dispatch (ev, (ODEventData *) event);
	}
	return handled;
}

//------------------------------------------------------------------------------

ODBoolean CPVideoPart::PrivHandleMouseDown(Environment* ev, ODEventData* event,
										 ODFrame* frame, ODFacet* facet)
{
	EnteringMethod("\pCPVideoPart::PrivHandleMouseDown");

	ODBoolean handled = kODFalse;

	if (facet == kODNULL)		// Did we get a click outside a modal dialog? 
	{
		::SysBeep(1);
		return kODTrue;			// Return that click was handled.
	}

	// Activate the window if it is not active
	if (! facet->GetWindow(ev)->IsActive(ev))
	{
		// select the window
		facet->GetWindow(ev)->Select(ev);

		// typically you continue handling the event, if not, just return kODTrue
		handled = kODTrue;
		
		// return kODTrue; // user needs to click twice to get something done
	}

	// dialog events first
	if (::IsDialogEvent((EventRecord *)event))
	{
		short		itemHit;
		DialogPtr	dialog;

		handled = ::DialogSelect((EventRecord *)event, &dialog, &itemHit);
		ODWindow* window = fSession->GetWindowState(ev)->GetODWindow(ev, dialog);
		return handled;		// In preparation for more "if" cases to be added below.
	}

	// get the mouse in local coordinates
	ODWindow* odwnd = frame->GetWindow(ev);
	ODPoint odPoint;
	GetWindowPoint(odwnd, ev, event->where, &odPoint);

	// test for click in grow box, if appropriate
	if (frame->IsRoot(ev))
	{
		WindowPtr wnd   = (WindowPtr)odwnd->GetPlatformWindow(ev);

		Rect r = wnd->portRect;
		r.left = r.right  - 15;
		r.top  = r.bottom - 15;
		if (::PtInRect(odPoint.AsQDPoint(), &r)) {
			::SetRect(&r, 200, 200, 20000, 20000);
			long newWindowSize = ::GrowWindow(wnd, event->where, &r);
			::SizeWindow(wnd, LoWord(newWindowSize), HiWord(newWindowSize), true);
			odwnd->AdjustWindowShape(ev);
			return kODTrue;
		}
	}

	if ( fSession->GetArbitrator(ev)->RequestFocusSet(ev, fFocusSet,frame) )
	{
		// Try to grab all of our foci
		this->FocusAcquired(ev, fSelectionFocus, frame);
		this->FocusAcquired(ev, fMenuFocus, frame);
		this->FocusAcquired(ev, fKeyFocus, frame);

	} else {
		// We couldn't get focus we requested, so bail.
		// Alternatively, parts of functionality can be disabled.
		return kODFalse;
	}

	// Handle clicks in used shape
	odPoint = facet->GetWindowContentTransform(ev, kODNULL)->InvertPoint(ev, &odPoint);

	ODShape*  usedShape = this->PrivGetUsedShape(ev, frame);
	ODBoolean hit       = usedShape->ContainsPoint(ev, &odPoint);
	usedShape->Release(ev);

	if (hit)
		handled = this->PrivUsedShapeHandleMouseDown(ev, event, frame, facet);

	return handled;
}

//------------------------------------------------------------------------------

ODBoolean CPVideoPart::PrivUsedShapeHandleMouseDown(Environment* ev,
		ODEventData* event, ODFrame* frame, ODFacet* facet)
{
	EnteringMethod("\pCPVideoPart::PrivUsedShapeHandleMouseDown");

	// Get the window and platform window (i.e. WindowPtr)
	ODWindow* odwnd = frame->GetWindow(ev);
	ODPlatformWindow facetPort = odwnd->GetPlatformWindow(ev);

	// Get the mouse in window-local and facet-local coordinates
	ODPoint windowPoint;
	GetWindowPoint(odwnd, ev, event->where, &windowPoint);
	ODPoint facetPoint = 
		facet->GetWindowContentTransform(ev, kODNULL)->InvertPoint(ev, &windowPoint);

	// Handle the click in whatever way is appropriate
	{
		CFocus  foc(ev, facet);
		Point	macPoint = facetPoint.AsQDPoint();
		// You now have the click point in facet coordinate space, so do something.
	}

	return kODTrue;		// We handled the click.
}