//----------------------------------------------------------------------------------------
// Frame protocol
//----------------------------------------------------------------------------------------

#ifndef SOM_ODDispatcher_xh
#include <Disptch.xh>
#endif
#ifndef _VIDEOPART_
#include "CPVideoPart.h"
#endif

//----------------------------------------------------------------------------------------

void CPVideoPart::DisplayFrameAdded(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisplayFrameAdded
	EnteringMethod("\pCPVideoPart::AddDisplayFrame");

	if (fSequenceGrabber)
		fSession->GetDispatcher (ev)->RegisterIdle (ev, fSelf, frame, 1);

	fDisplayFrames.Add(frame);
		// Add new frame to list of current display frames.
	
}

//----------------------------------------------------------------------------------------

void CPVideoPart::DisplayFrameConnected(Environment* ev, ODFrame* /*frame*/)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisplayFrameConnected
	EnteringMethod("\pCPVideoPart::DisplayFrameConnected");

	// // fDisplayFrames.Add(frame);
	// Don't need to add frame to list of current display frames because we already did
	// in InitPartFromStorage.
}

//----------------------------------------------------------------------------------------

void CPVideoPart::DisplayFrameRemoved(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisplayFrameRemoved
	EnteringMethod("\pCPVideoPart::DisplayFrameRemoved");

	if (fSequenceGrabber)
		fSession->GetDispatcher (ev)->UnregisterIdle (ev, fSelf, frame);

	fSession->GetArbitrator(ev)->RelinquishFocusSet(ev, fFocusSet, frame);

	fDisplayFrames.Remove(frame);
		// This just removes the reference from our frame list.
}

//----------------------------------------------------------------------------------------

void CPVideoPart::DisplayFrameClosed(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisplayFrameClosed
	EnteringMethod("\pCPVideoPart::DisplayFrameClosed");

	if (fSequenceGrabber)
		fSession->GetDispatcher (ev)->UnregisterIdle (ev, fSelf, frame);

	fSession->GetArbitrator(ev)->RelinquishFocusSet(ev, fFocusSet, frame);

	fDisplayFrames.Remove(frame);
		// This just removes the reference from our frame list.
}

//----------------------------------------------------------------------------------------

void CPVideoPart::FrameShapeChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FrameShapeChanged
	EnteringMethod("\pCPVideoPart::FrameShapeChanged");

	frame->RequestFrameShape(ev, this->PrivGetFrameShape(ev, frame), kODNULL);
	this->PrivAdjustFacets(ev, frame);
	frame->Invalidate(ev, kODNULL, kODNULL);
	frame->InvalidateActiveBorder(ev);
}

//----------------------------------------------------------------------------------------

ODShape* CPVideoPart::PrivGetFrameShape(Environment* ev, ODFrame* frame)
{
	// This is just to centralize the frame shape code.  Do it any way
	// is best for your part.

	ODShape* newFrameShape;

	if (frame->IsRoot(ev))
		newFrameShape = frame->GetFrameShape(ev, kODNULL);
	else
		newFrameShape = this->PrivGetUsedShape(ev, frame);

	return newFrameShape;
}

//----------------------------------------------------------------------------------------

ODShape* CPVideoPart::PrivGetUsedShape(Environment* ev, ODFrame* frame)
{
	// This is just to centralize the frame shape code.  Do it any way
	// is best for your part.
	
	ODShape* frameShape = frame->GetFrameShape(ev, kODNULL);
	RgnHandle	shapeRgn = frameShape->GetQDRegion(ev);
	Rect		rr;

	frameShape->Release(ev);

	ODTypeToken	theViewType = frame->GetViewType(ev);
		
	// handle Small Icon view
	if (theViewType == fSession->Tokenize(ev, kODViewAsSmallIcon))
	{
		::SetRect(&rr, 0, 0, kODSmallIconSize, kODSmallIconSize);
	}
	// handle Large Icon view
	else if (theViewType == fSession->Tokenize(ev, kODViewAsLargeIcon))
	{
		::SetRect(&rr, 0, 0, kODLargeIconSize, kODLargeIconSize);
	}
	// handle Thumbnail view
	else if (theViewType == fSession->Tokenize(ev, kODViewAsThumbnail))
	{
		::SetRect(&rr, 0, 0, kODThumbnailSize, kODThumbnailSize);
	}
	// handle Frame viewType
	else
	{
		rr = (*shapeRgn)->rgnBBox;
		short	hh = rr.bottom - rr.top;
		short	ww = rr.right - rr.left;
	
		::SetRect(&rr, 0, 0, ww, hh);
	}

	::RectRgn(shapeRgn = NewRgn(), &rr);

	ODShape* usedShape = frame->CreateShape(ev);
	usedShape->SetQDRegion(ev, shapeRgn);

	return usedShape;
}

//----------------------------------------------------------------------------------------

void CPVideoPart::PrivAdjustFacets(Environment* ev, ODFrame* frame)
{
	// Given a frame, iterate through all of the facets and adjust them
	// according to the new frame.

	ODShape* usedShape  = this->PrivGetUsedShape(ev, frame);
	frame->ChangeUsedShape(ev, usedShape, kODNULL);
	usedShape->Release(ev);

	ODShape* frameShape = frame->GetFrameShape(ev, kODNULL);
	ODFrameFacetIterator* facets = frame->CreateFacetIterator(ev);
	for (ODFacet* facet = facets->First(ev);
			facets->IsNotComplete(ev);
			facet = facets->Next(ev) )
	{
		facet->ChangeActiveShape(ev, frameShape, kODNULL);
	}
	frameShape->Release(ev);
}


//----------------------------------------------------------------------------------------

void CPVideoPart::AttachSourceFrame(Environment* ev, ODFrame* frame, ODFrame* sourceFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_AttachSourceFrame
	EnteringMethod("\pCPVideoPart::AttachSourceFrame");
	ODUnused(frame);
	ODUnused(sourceFrame);
}

//----------------------------------------------------------------------------------------

void CPVideoPart::ViewTypeChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ViewTypeChanged
	EnteringMethod("\pCPVideoPart::ViewTypeChanged");

	// Invalidate the entire frame
	frame->Invalidate(ev, kODNULL, kODNULL);
	frame->InvalidateActiveBorder(ev);
}

//----------------------------------------------------------------------------------------

void CPVideoPart::PresentationChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_PresentationChanged
	EnteringMethod("\pCPVideoPart::PresentationChanged");
	ODUnused(frame);
}

//----------------------------------------------------------------------------------------

void CPVideoPart::SequenceChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_SequenceChanged
	EnteringMethod("\pCPVideoPart::SequenceChanged");
	ODUnused(frame);
}

//----------------------------------------------------------------------------------------

void CPVideoPart::WritePartInfo(Environment* ev, ODInfoType partInfo,
							   ODStorageUnitView* storageUnitView)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_WritePartInfo
	EnteringMethod("\pCPVideoPart::WritePartInfo");
	ODUnused(partInfo);
	ODUnused(storageUnitView);
}

//----------------------------------------------------------------------------------------

void CPVideoPart::ClonePartInfo(Environment *ev, ODDraftKey key,
								ODInfoType partInfo,
								ODStorageUnitView* storageUnitView,
								ODFrame* scopeFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ClonePartInfo
	EnteringMethod("\pCPVideoPart::ClonePartInfo");
	ODUnused(key);
	ODUnused(partInfo);
	ODUnused(storageUnitView);
	ODUnused(scopeFrame);
}

//----------------------------------------------------------------------------------------

ODInfoType CPVideoPart::ReadPartInfo(Environment* ev, ODFrame* frame, ODStorageUnitView* storageUnitView)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ReadPartInfo
	EnteringMethod("\pCPVideoPart::ReadPartInfo");
	ODUnused(frame);
	ODUnused(storageUnitView);
	return kODNULL;
}

//----------------------------------------------------------------------------------------

ODFrame* CPVideoPart::RequestEmbeddedFrame(Environment* ev, ODFrame* containingFrame,
										   ODFrame* baseFrame,
										   ODShape* frameShape,
										   ODPart* embedPart,
										   ODTypeToken viewType,
										   ODTypeToken presentation,
										   ODBoolean isOverlaid)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RequestEmbeddedFrame
	EnteringMethod("\pCPVideoPart::RequestEmbeddedFrame");
	ODUnused(containingFrame);
	ODUnused(baseFrame);
	ODUnused(frameShape);
	ODUnused(embedPart);
	ODUnused(viewType);
	ODUnused(presentation);
	ODUnused(isOverlaid);
	DebugStr("\pCPVideoPart::RequestEmbeddedFrame -- shouldn't be here.");
	return kODNULL;
}

//----------------------------------------------------------------------------------------

void CPVideoPart::RemoveEmbeddedFrame(Environment* ev, ODFrame* embeddedFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RemoveEmbeddedFrame
	EnteringMethod("\pCPVideoPart::RemoveEmbeddedFrame");
	ODUnused(embeddedFrame);
	DebugStr("\pCPVideoPart::RemoveEmbeddedFrame -- shouldn't be here.");
}

//----------------------------------------------------------------------------------------

ODShape* CPVideoPart::RequestFrameShape(Environment* ev, ODFrame* embeddedFrame, ODShape* frameShape)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RequestFrameShape
	EnteringMethod("\pCPVideoPart::RequestFrameShape");
	ODUnused(embeddedFrame);
	ODUnused(frameShape);
	DebugStr("\pCPVideoPart::RequestFrameShape -- shouldn't be here.");
	return kODNULL;
}

//----------------------------------------------------------------------------------------

void CPVideoPart::UsedShapeChanged(Environment* ev, ODFrame* embeddedFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_UsedShapeChanged
	EnteringMethod("\pCPVideoPart::UsedShapeChanged");
	ODUnused(embeddedFrame);
}

//----------------------------------------------------------------------------------------

ODShape* CPVideoPart::AdjustBorderShape(Environment* ev, ODFacet* embeddedFacet, ODShape* shape)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_AdjustBorderShape
	EnteringMethod("\pCPVideoPart::AdjustBorderShape");
	ODUnused(embeddedFacet);
	return shape;
}
