//----------------------------------------------------------------------------------------
// Imaging protocol
//----------------------------------------------------------------------------------------

#ifndef _VIDEOPART_
#include "CPVideoPart.h"
#endif

#ifndef __ICONS__
#include <Icons.h>
#endif

#if GENERATING68K
#pragma segment Imaging
#endif

//----------------------------------------------------------------------------------------

void CPVideoPart::Draw (Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Draw
	EnteringMethod ("\pCPVideoPart::Draw");
	ODUnused (invalidShape);

	CFocus	foc (ev, facet);

	ODTypeToken view = facet->GetFrame (ev)->GetViewType (ev);

	if (view == fSession->Tokenize (ev, kODViewAsLargeIcon))
	{
		this->PrivDrawAsLargeIcon (ev, facet, invalidShape);
		return;
	}
	
	if (view == fSession->Tokenize (ev, kODViewAsSmallIcon))
	{
		this->PrivDrawAsSmallIcon (ev, facet, invalidShape);
		return;
	}
	
	if (view == fSession->Tokenize (ev, kODViewAsThumbnail))
	{
		this->PrivDrawAsThumbnail (ev, facet);
		return;
	}
	
	if (view == fSession->Tokenize (ev, kODViewAsFrame))
	{
		ODFrame *frame;
		ODShape *usedShape;
		RgnHandle newClip, oldClip, usedRegion;
		Rect usedRect;
	
		CFocus foc (ev, facet);
	
		frame = facet->GetFrame (ev);
	
		//	Get the used rectangle.
	
		usedShape = frame->GetUsedShape (ev, kODNULL);
		usedRegion = usedShape->GetQDRegion (ev);
		usedShape->Release (ev);
		usedRect = (*usedRegion)->rgnBBox;
	
		//	If were the root, draw the grow icon and erase the rest of the window.
		//	Otherwise, just erase the used (round) rect of this frame.
	
		oldClip = nil;
		if (frame->IsRoot (ev)) {
			GrafPtr	curPort;
	
			::GetPort (&curPort);
	
			Rect r = curPort->portRect;
			r.left = r.right  - 15;
			r.top  = r.bottom - 15;
			::GetClip (oldClip = NewRgn ());
			::ClipRect (&r);
			::DrawGrowIcon (curPort);
			::GetClip (newClip = ::NewRgn ());
			::DiffRgn (oldClip, newClip, newClip);
			::SetClip (newClip);
			::DisposeRgn (newClip);
			::EraseRect (&(curPort->portRect));
		}
		else {
			::EraseRoundRect (&usedRect, 40, 40);
		}
		
		//	Now actually draw the content.
		
		this->PrivDrawContent (ev, facet);
			
		//	Restore the clip region.
			
		if (oldClip) {
			::SetClip (oldClip);
			::DisposeRgn (oldClip);
		}
	}
}

//----------------------------------------------------------------------------------------

static pascal OSErr PrivDetachIconResource (ResType theType, Handle *theIcon, void* yourDataPtr)
{
	if (theIcon != kODNULL && *theIcon != kODNULL)
	{
		::DetachResource (*theIcon);
		return ::ResError ();
	}
	else
		return noErr;
}

//----------------------------------------------------------------------------------------

void CPVideoPart::PrivDrawAsLargeIcon (Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	Rect				rect32;
	IconTransformType 	transformType = ttNone;
	Handle				iconSuite;
	
	// Check to see if the icon is selected and in an active window
	if ( facet->GetHighlight (ev) == kODFullHighlight && facet->GetWindow (ev)->IsActive (ev) )
		transformType = ttSelected;
	
	// Draw the icon.
	{
		CUsingLibraryResources res;
		OSErr error = ::GetIconSuite (&iconSuite, kHelloIconSuite, svAllLargeData);		
	
		if ( error == noErr )
		{
			// Create a universal proc pointer for our icon action proc.
			IconActionUPP actionProc = NewIconActionProc (PrivDetachIconResource);
			error = ::ForEachIconDo (iconSuite, svAllLargeData, actionProc, kODNULL);
			DisposeRoutineDescriptor (actionProc);
		}
	}
	
	::SetRect (&rect32, 0, 0, kODLargeIconSize, kODLargeIconSize);
	::PlotIconSuite (&rect32, atAbsoluteCenter, transformType, iconSuite);

	if ( facet->GetHighlight (ev) == kODDimHighlight )
	{
		ODShape* usedShape = facet->GetFrame (ev)->GetUsedShape (ev, kODNULL);
		RgnHandle usedRgn = usedShape->GetQDRegion (ev);
		
		::PenPat (&ODQDGlobals.gray);
		::FrameRgn (usedRgn);
		::PenNormal ();
		
		usedShape->Release (ev);
	}
}

//----------------------------------------------------------------------------------------

void CPVideoPart::PrivDrawAsSmallIcon (Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	Rect				rect16;
	IconTransformType 	transformType = ttNone;
	Handle				iconSuite;
	
	// Check to see if the icon is selected and in an active window
	if ( facet->GetHighlight (ev) == kODFullHighlight && facet->GetWindow (ev)->IsActive (ev) )
		transformType = ttSelected;
	
	// Draw the icon.
	{
		CUsingLibraryResources res;
		OSErr error = ::GetIconSuite (&iconSuite, kHelloIconSuite, svAllSmallData);		
	
		if ( error == noErr )
		{
			// Create a universal proc pointer for our icon action proc.
			IconActionUPP actionProc = NewIconActionProc (PrivDetachIconResource);
			error = ::ForEachIconDo (iconSuite, svAllLargeData, actionProc, kODNULL);
			DisposeRoutineDescriptor (actionProc);
		}
	}
	
	::SetRect (&rect16, 0, 0, kODSmallIconSize, kODSmallIconSize);
	::PlotIconSuite (&rect16, atAbsoluteCenter, transformType, iconSuite);

	if ( facet->GetHighlight (ev) == kODDimHighlight )
	{
		ODShape* usedShape = facet->GetFrame (ev)->GetUsedShape (ev, kODNULL);
		RgnHandle usedRgn = usedShape->GetQDRegion (ev);
		
		::PenPat (&ODQDGlobals.gray);
		::FrameRgn (usedRgn);
		::PenNormal ();
		
		usedShape->Release (ev);
	}
}

//----------------------------------------------------------------------------------------

void CPVideoPart::PrivDrawAsThumbnail (Environment* ev, ODFacet* facet)
{
	if (fSequenceGrabber) {
		Rect thumbnailRect;
		GrafPtr	currentGrafPort;
		
		//	Get the current GrafPort and draw the image in it as a thumbnail.
	
		::GetPort (&currentGrafPort);
		::SetRect (&thumbnailRect, 0, 0, kODThumbnailSize, kODThumbnailSize);
		::CopyBits ((BitMap *) *fOffscreenPixMap, &currentGrafPort->portBits, &fOffscreenRect,
					&thumbnailRect, ditherCopy, nil);
	}
}

//----------------------------------------------------------------------------------------

void CPVideoPart::PrivDrawContent (Environment *ev, ODFacet *facet)
{
	ODShape *usedShape;
	GrafPtr	currentGrafPort;
	RgnHandle usedRegion;
	Rect usedRect;

	//	Get the current GrafPort.

	::GetPort (&currentGrafPort);

	//	Get the used rectangle.

	usedShape = facet->GetFrame (ev)->GetUsedShape (ev, kODNULL);
	usedRegion = usedShape->GetQDRegion (ev);
	usedShape->Release (ev);
	usedRect = (*usedRegion)->rgnBBox;

	//	Draw the content.
	
	if (fSequenceGrabber) {
		usedRect.right = usedRect.left + fOffscreenRect.right;
		usedRect.bottom = usedRect.top + fOffscreenRect.bottom;
		::CopyBits ((BitMap *) *fOffscreenPixMap, &currentGrafPort->portBits, &fOffscreenRect,
					&usedRect, srcCopy, nil);
	}
	else {
		unsigned char noVideoString [255];
		
		{
			CUsingLibraryResources fil;

			::GetIndString (noVideoString, kSTRx_ErrorStrings, kNoVideoString);
		}
		::MoveTo ((usedRect.right - usedRect.left - ::StringWidth (noVideoString)) / 2,
				  (usedRect.bottom - usedRect.top) / 2 + 5);
		::DrawString (noVideoString);
	}
}

//----------------------------------------------------------------------------------------

void CPVideoPart::CanvasUpdated (Environment* ev, ODCanvas* canvas)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CanvasUpdated
	EnteringMethod ("\pCPVideoPart::CanvasUpdated");
	ODUnused (canvas);
}

//----------------------------------------------------------------------------------------

void CPVideoPart::HighlightChanged (Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_HighlightChanged
	EnteringMethod ("\pCPVideoPart::HighlightChanged");
	ODUnused (facet);
}

//----------------------------------------------------------------------------------------

ODULong CPVideoPart::GetPrintResolution (Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_GetPrintResolution
	EnteringMethod ("\pCPVideoPart::GetPrintResolution");
	ODUnused (frame);
	return 0;
}

//----------------------------------------------------------------------------------------

void CPVideoPart::FacetAdded (Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FacetAdded
	EnteringMethod ("\pCPVideoPart::FacetAdded");

	ODFrame* frame = facet->GetFrame (ev);
	frame->RequestFrameShape (ev, this->PrivGetFrameShape (ev, frame), kODNULL);
		// First, PrivGetFrameShape is called to calculate the frame
		// shape we want.  However, we might not get it, due to the
		// needs/constraints of the containing part.  Therefore, after
		// calculating what we want, we see if we can get it by calling
		// RequestFrameShape.
		// RequestFrameShape either returns us what we passed it, or
		// returns something else because it thinks it knows best.
		// Either way, we then proceed with the returned result.
		// Note that this means that we have to be prepared to deal
		// with a frame shape other than we desire.

	this->PrivAdjustFacets (ev, frame);
		// Given a new negotiated frame shape, adjust our facets based
		// on it.  That's all there is to it.

	if (frame->IsRoot (ev))
	{
		ODBoolean succeeded = kODFalse;
		succeeded = fSession->GetArbitrator (ev)->RequestFocusSet (ev, fFocusSet,frame);
		if (succeeded)
		{
			this->FocusAcquired (ev, fSelectionFocus, frame);
			this->FocusAcquired (ev, fMenuFocus, frame);
			this->FocusAcquired (ev, fKeyFocus, frame);
		}
	}
}

//----------------------------------------------------------------------------------------

void CPVideoPart::FacetRemoved (Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FacetRemoved
	EnteringMethod ("\pCPVideoPart::FacetRemoved");
}

//----------------------------------------------------------------------------------------

void CPVideoPart::CanvasChanged (Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CanvasChanged
	EnteringMethod ("\pCPVideoPart::CanvasChanged");
	ODUnused (facet);
}

//----------------------------------------------------------------------------------------

void CPVideoPart::GeometryChanged (Environment* ev, ODFacet* facet,
												   ODBoolean clipShapeChanged,
												   ODBoolean externalTransformChanged)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_GeometryChanged
	EnteringMethod ("\pCPVideoPart::GeometryChanged");
	ODUnused (facet);
	ODUnused (clipShapeChanged);
	ODUnused (externalTransformChanged);
}